## Copyright (C) 2010 Kishan Goyal <kishan@fedoraproject.org>
## Copyright (C) 2010 Meejanur Rahaman <realmeejan@gmail.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import os
import sys

import pygtk

import gtk
import gtk.glade
import gobject

from photo import *
from matplotlib.figure import Figure
from matplotlib.backends.backend_gtk import FigureCanvasGTK as FigureCanvas
import numpy as np
from pylab import show


class Gfotostat:
    """ Test Program"""
    def __init__(self):
        gobject.threads_init()
        self.dirname = ''
        self.toggle = None
        self.choice = None
        self.count = {}
        self.total = {}
        self.totals_fl = {}
        self.totals_ap = {}
        self.totals_st = {}
        self.f = None
        self.wTree = gtk.glade.XML('gfotostat.glade')
        self.window = self.wTree.get_widget('MainWindow')
        self.typebox = self.wTree.get_widget('typebox')
        self.genBttn = self.wTree.get_widget('runBttn')
        self.genBttn.set_sensitive(False)
        self.statusbar = self.wTree.get_widget('statusbar1')
        self.togglebutton1 = self.wTree.get_widget('togglebutton1')
        self.scw = self.wTree.get_widget('scw')

        self.window.set_size_request(800, 600)
        self.window.set_icon_from_file("/usr/share/pixmaps/gfotostat.png")
        self.typebox.set_active(0)
        self.window.show_all()

        cbs = {'on_MainWindow_destroy':gtk.main_quit,
               'on_browseBttn_clicked':self.browseBttn_cb,
               'on_open1_activate':self.browseBttn_cb,
               'on_about_activate':self.show_about,
               'on_runBttn_clicked':self.runBttn_cb,
               'on_quit1_activate':gtk.main_quit,
               'on_typebox_changed':self.combo_cb,
               'on_save_as1_activate':self.FileSaveAsMenu,
               'on_togglebutton1_toggled':self.toggle_cb}
        self.wTree.signal_autoconnect(cbs)


    def statusbar_cb(self, widget):
        """
        To display the total number of files in the Statusbar
        """

        if 'browseBttn_clicked':
            self.count = None
            self.statusbar.push(1, "Please click Generate.")
            self.statusbar.show()
            self.genBttn.set_sensitive(True)

     #   if 'runBttn_clicked':
      #      self.statusbar.push( 1, "Total number of files: " + self.count )
       #     self.statusbar.show()


    def savefigure(self):
        """ Saves the images"""
        self.f.savefig('stats.png')


    def browseBttn_cb(self, widget):
        """
        Select the folder
        """

        chooser = gtk.FileChooserDialog(title='Select Directory',
        action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,
        gtk.RESPONSE_OK))
        response = chooser.run()

        if response == gtk.RESPONSE_OK:
            self.dirname = chooser.get_filename()
        chooser.destroy()
        print self.dirname
        self.statusbar_cb(self)

    def FileSaveAsMenu(self, widget):
        chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_SAVE,
        buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OK,gtk.RESPONSE_OK))
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            filename = chooser.get_filename()
            print filename
            self.f.savefig(filename)
        chooser.destroy()

    def show_about(self):

        """
        Show the about dialog
        """

        dialog = gtk.AboutDialog()
        dialog.set_name('gFotoStat')
        dialog.set_copyright('(c) 2010 Kishan Goyal, Meejanur Rahaman')
        dialog.set_website('http://gitorious.org/gfotostat')
        dialog.set_authors(['Kishan Goyal <fabhoneydew@gmail.com>', 'Meejanur Rahaman <realmeejan@gmail.com>'])
        dialog.set_program_name('gFotoStat')
        dialog.run()
        dialog.destroy()

    def toggle_cb(self, widget):
        """
        Toggle between pie and bar charts
        """

        self.toggle = self.togglebutton1.get_active()
        if 'runBttn_clicked':
            if self.toggle == True:
                self.togglebutton1.set_label("Toggle")
                self.bar_blocks()
            elif self.toggle == False:
                self.togglebutton1.set_label("Toggle")
                self.pie_blocks()

    def combo_cb(self, widget):
        """
        Generate different charts
        """

        model = self.typebox.get_model()
        index = self.typebox.get_active()
        self.choice =  model[index][0]

        if 'runBttn_clicked':
            self.toggle_cb(widget)

    def bar_blocks(self):

        """
        Generate bar charts
        """

        if self.scw.get_child():
            self.scw.remove(self.scw.get_child())
        if self.choice == 'ISO':
            self.plot_bar_iso()
        elif self.choice == 'Aperture':
            self.plot_bar_ap()
        elif self.choice == 'Focal Length':
            self.plot_bar_fl()
        elif self.choice == 'Shutter Speed':
            self.plot_bar_st()
        else:
            self.plot_bar_overview()


    def pie_blocks(self):

        """
        Generate pie charts
        """

        if self.scw.get_child():
            self.scw.remove(self.scw.get_child())
        if self.choice == 'ISO':
            self.plot_pie_iso()
        elif self.choice == 'Aperture':
            self.plot_pie_ap()
        elif self.choice == 'Focal Length':
            self.plot_pie_fl()
        elif self.choice == 'Shutter Speed':
            self.plot_pie_st()
        else:
            self.plot_pie_overview()


    def runBttn_cb(self, widget):
        
        """
        Display info about images
        """

        files = get_images(self.dirname)
        for f in files:
            try:
                result = get_exif(f)
            except IOError, e:
                print e
                continue
            if result:
                iso = result['iso']
                focallength = result['focallength']
                aperture = result['aperture']
                sutterspeed = result['sutterspeed']
                if iso in self.total:
                    x = self.total[iso]
                    x += 1
                    self.total[iso] = x
                else:
                    self.total[iso] = 1

                if focallength in self.totals_fl:
                    z = self.totals_fl[focallength]
                    z += 1
                    self.totals_fl[focallength] = z
                else:
                    self.totals_fl[focallength] =1

                if aperture in self.totals_ap:
                    y = self.totals_ap[aperture]
                    y += 1
                    self.totals_ap[aperture] = y
                else:
                    self.totals_ap[aperture] = 1

                if sutterspeed in self.totals_st:
                    y = self.totals_st[sutterspeed]
                    y += 1
                    self.totals_st[sutterspeed] = y
                else:
                    self.totals_st[sutterspeed] = 1
        if self.scw.get_child():
            self.scw.remove(self.scw.get_child())
        self.typebox.set_active(0)
        self.togglebutton1.set_active(0)
        self.plot_bar_overview()
        self.genBttn.set_sensitive(False)


    def plot_bar_overview(self):
 
        """
        plot overview of bar charts
        """

        count = sum(self.total.values())
        self.count = str(count)
        self.statusbar.push( 1, "Total number of files: " + self.count )
        self.statusbar.show()
       # count = sum(self.total.values())
       # self.count = str(count)
        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.total.keys()]
        count = sum(self.total.values())
        fracs = [ (val * 100) / float(count) for val in self.total.values()]
        width = 0.35
        i = len(labels)
        ind = np.arange(i)
        ax1 = self.f.add_subplot(221)
        ax1.bar(ind, fracs, color = 'r')
        ax1.set_ylabel('Percentage of Images')
       # ax1.set_xlabel('Values range')
        ax1.set_title('ISO', bbox = {'facecolor':'0.8', 'pad':5})
        ax1.set_xticks(ind + width )
        ax1.set_xticklabels(labels)


        labels_ap = [key for key in self.totals_ap.keys()]
        count_ap = sum(self.totals_ap.values())
        fracs_ap = [ (val * 100) / float(count_ap) for val in self.totals_ap.values()]
        i_ap = len(labels_ap)
        ind = np.arange(i_ap)
        ax2 = self.f.add_subplot(222)
        ax2.bar(ind, fracs_ap, color = 'b')
        ax2.set_ylabel('Percentage of Images')
       # ax2.set_xlabel('Aperture Values range')
        ax2.set_title('Aperture', bbox = {'facecolor':'0.8', 'pad':5})
        ax2.set_xticks(ind + width )
        ax2.set_xticklabels(labels_ap)


        labels_fl = [key for key in self.totals_fl.keys()]
        count_fl = sum(self.totals_fl.values())
        fracs_fl = [ (val * 100) / float(count_fl) for val in self.totals_fl.values()]
        i_fl = len(labels_fl)
        ind = np.arange(i_fl)
        ax3 = self.f.add_subplot(223)
        ax3.bar(ind, fracs_fl, color = 'g')
        ax3.set_ylabel('Percentage of Images')
       # ax3.set_xlabel('Foculength Values range')
        ax3.set_title('Focal Length', bbox = {'facecolor':'0.8', 'pad':5})
        ax3.set_xticks(ind + width )
        ax3.set_xticklabels(labels_fl)


        labels_st = [key for key in self.totals_st.keys()]
        count_st = sum(self.totals_st.values())
        fracs_st = [ (val * 100) / float(count_st) for val in self.totals_st.values()]
        i_st = len(labels_st)
        ind = np.arange(i_st)
        ax4 = self.f.add_subplot(224)
        ax4.bar(ind, fracs_st, color = 'b')
        ax4.set_ylabel('Percentage of Images')
       # ax4.set_xlabel('Shutter values range')
        ax4.set_title('Shutter Speed', bbox = {'facecolor':'0.8', 'pad':5})
        ax4.set_xticks(ind + width )
        ax4.set_xticklabels(labels_st)

        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()

    def plot_bar_iso(self):
        """
        Plot bar charts of ISO
        """

        f = Figure(figsize = (5, 5))
        labels = [key for key in self.total.keys()]
        print labels
        count = sum(self.total.values())
        fracs = [ (val * 100) / float(count) for val in self.total.values()]
        width = 0.35
        i = len(labels)
        ind = np.arange(i)
        ax1 = self.f.add_subplot(111)
        abc = ax1.bar(ind, fracs, color = 'b')
        ax1.set_ylabel('Percentage of Images')
        ax1.set_xlabel('ISO')
        ax1.set_title(' ISO Plot', bbox = {'facecolor':'0.8', 'pad':5})
        ax1.set_xticks(ind + width )
        ax1.set_xticklabels(labels)
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()



    def plot_bar_fl(self):
        """
        Plot bar charts of focal Length
        """
 
        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.totals_fl.keys()]
        count = sum(self.totals_fl.values())
        fracs = [ (val * 100) / float(count) for val in self.totals_fl.values()]
        width = 0.35
        i = len(labels)
        ind = np.arange(i)
        ax1 = self.f.add_subplot(111)
        abc = ax1.bar(ind, fracs, color = 'b')
        ax1.set_ylabel('Percentage of Images')
        ax1.set_xlabel('Focal Length')
        ax1.set_title('Focal Length Plot',bbox = {'facecolor':'0.8', 'pad':5})
        ax1.set_xticks(ind + width )
        ax1.set_xticklabels(labels)
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()

    def plot_bar_ap(self):
        """
        Plot bar charts of aperture values
        """
        
        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.totals_ap.keys()]
        count = sum(self.totals_ap.values())
        fracs = [ (val * 100) / float(count) for val in self.totals_ap.values()]
        width = 0.35
        i = len(labels)
        ind = np.arange(i)
        ax1 = self.f.add_subplot(111)
        abc = ax1.bar(ind, fracs, color = 'b')
        ax1.set_ylabel('Percentage of Images')
        ax1.set_xlabel('Aperture')
        ax1.set_title('Aperture Plot', bbox = {'facecolor':'0.8', 'pad':5})
        ax1.set_xticks(ind + width )
        ax1.set_xticklabels(labels)
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()

    def plot_bar_st(self):
        """
        Plot bar charts of shutterspeed
        """
        
        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.totals_st.keys()]
        count = sum(self.totals_st.values())
        fracs = [ (val * 100) / float(count) for val in self.totals_st.values()]
        width = 0.35
        i = len(labels)
        ind = np.arange(i)
        ax1 = self.f.add_subplot(111)
        abc = ax1.bar(ind, fracs, color = 'b')
        ax1.set_ylabel('Percentage of Images')
        ax1.set_xlabel('Shutter Speed')
        ax1.set_title('Shutter Speed Plot', bbox = {'facecolor':'0.8', 'pad':5})
        ax1.set_xticks(ind + width )
        ax1.set_xticklabels(labels)
        #plt.show()
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()


    def plot_pie_overview(self):
        """
        Plot the pie-charts for focal length, shutter speed, Aperture and ISO values
        """

        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.total.keys()]
        count = sum(self.total.values())
 #       self.count = str(count)
        fracs = [ (val * 100) / float(count) for val in self.total.values() ]
        ax1 = self.f.add_axes( [0.05, 0.4, 0.4, 0.4] )
        pie1 = ax1.pie( fracs, labels = labels, autopct = '%1.1f%%')
        ax1.set_title('ISO', bbox = {'facecolor':'0.8', 'pad':5})
        self.statusbar_cb(self)

        labels_ap = [key for key in self.totals_ap.keys()]
        count_ap = sum(self.totals_ap.values())
        fracs_ap = [ (val * 100) / float(count_ap) for val in self.totals_ap.values()]
        ax2 = self.f.add_axes([0.55, 0.4, 0.4, 0.4])
        pie2 = ax2.pie(fracs_ap, labels = labels_ap, autopct = '%1.1f%%')
        ax2.set_title('Aperture', bbox = {'facecolor':'0.8', 'pad':5})

        labels_fl = [key for key in self.totals_fl.keys()]
        count_fl = sum(self.totals_fl.values())
        fracs_fl = [ (val * 100) / float(count_fl) for val in self.totals_fl.values()]
        ax3 = self.f.add_axes([0.05, 0.0, 0.4, 0.4])
        pie3 = ax3.pie(fracs_fl, labels = labels_fl, autopct = '%1.1f%%')
        ax3.set_title('Focal Length', bbox = {'facecolor':'0.8', 'pad':5})

        labels_st = [key for key in self.totals_st.keys()]
        count_st = sum(self.totals_st.values())
        fracs_st = [ (val * 100) / float(count_st) for val in self.totals_st.values()]
        ax4 = self.f.add_axes([0.55, 0.0, 0.4, 0.4])
        pie4 = ax4.pie(fracs_st, labels = labels_st, autopct = '%1.1f%%')
        ax4.set_title('Shutter Speed', bbox = {'facecolor':'0.8', 'pad':5})
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()


    def plot_pie_iso(self):
        """
        Display a pie-chart for ISO values
        """

        self.f = Figure(figsize = (5, 5))
        labels = [key for key in self.total.keys()]
        count = sum(self.total.values())
        fracs = [ (val * 100) / float(count) for val in self.total.values()]
        ax1 = self.f.add_axes([0.1, 0.1, 0.8, 0.8])
        pie1 = ax1.pie(fracs, labels = labels, autopct = '%1.1f%%')
        ax1.set_title('ISO', bbox = {'facecolor':'0.8', 'pad':5})
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()


    def plot_pie_ap(self):
        """
        Display a pie-chart for aperture values
        """

        self.f = Figure(figsize = (5, 5))
        labels_ap = [key for key in self.totals_ap.keys()]
        count_ap = sum(self.totals_ap.values())
        fracs_ap = [ (val * 100) / float(count_ap) for val in self.totals_ap.values()]
        ax3 = self.f.add_axes([0.1, 0.1, 0.8, 0.8])
        pie3 = ax3.pie(fracs_ap, labels = labels_ap, autopct = '%1.1f%%')
        ax3.set_title('Aperture', bbox = {'facecolor':'0.8', 'pad':5})
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()


    def plot_pie_fl(self):
        """
        Display a pie-chart for focallength values
        """

        self.f = Figure( figsize =(5, 5))
        labels_fl = [key for key in self.totals_fl.keys()]
        count_fl = sum(self.totals_fl.values())
        fracs_fl = [ (val * 100) / float(count_fl) for val in self.totals_fl.values()]
        ax3 = self.f.add_axes([0.1, 0.1, 0.8, 0.8])
        pie3 = ax3.pie(fracs_fl, labels = labels_fl, autopct = '%1.1f%%')
        ax3.set_title('Focal Length', bbox = {'facecolor':'0.8', 'pad':5})
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()



    def plot_pie_st(self):
        """
        Display a pie-chart for sutterspeed values
        """

        self.f = Figure(figsize = (5, 5))
        labels_st = [key for key in self.totals_st.keys()]
        count_st = sum(self.totals_st.values())
        fracs_st = [ (val * 100) / float(count_st) for val in self.totals_st.values()]
        ax3 = self.f.add_axes([0.1, 0.1, 0.8, 0.8])
        pie3 = ax3.pie(fracs_st, labels = labels_st, autopct = '%1.1f%%')
        ax3.set_title('Shutter Speed', bbox = {'facecolor':'0.8', 'pad':5})
        canvas = FigureCanvas(self.f)  # a gtk.DrawingArea
        canvas.set_size_request(800, 600)
        self.scw.add_with_viewport(canvas)
        self.window.show_all()


if __name__ == '__main__':

    obj = Gfotostat()
    gtk.main()

